#ifndef simplicialcomplex_h_ooDeimaexieghaev
#define simplicialcomplex_h_ooDeimaexieghaev

#include <cpptest.h>
#include <cpptest-suite.h>

#include <libstick-0.1/simplicialcomplex.h>

using namespace libstick;


class simplicial_complex_TestSuite: public Test::Suite {

    private:
        typedef simplicial_complex<3, uint32_t, double> scomplex;
        typedef scomplex::simplex_order::boundary_matrix bm;

        bool setupcalled;
        scomplex c1, c2, c3;
        scomplex::simplex_order o1, o2, o3, o3b;

    public:
        simplicial_complex_TestSuite() :
            setupcalled(false),
            o1(c1),
            o2(c2),
            o3(c3),
            o3b(c3)
            {
            TEST_ADD(simplicial_complex_TestSuite::test_is_complex);
            TEST_ADD(simplicial_complex_TestSuite::test_is_monotoneComplex);
            TEST_ADD(simplicial_complex_TestSuite::test_is_order_filtration);
            TEST_ADD(simplicial_complex_TestSuite::test_is_order_monotone);
            TEST_ADD(simplicial_complex_TestSuite::test_boundary_matrix);
        }

    protected:
        virtual void setup() {
            if (setupcalled)
                return;
            setupcalled = true;

            scomplex::simplex ss[] = {
                // dimension, faces, value...
                {0, {}, 1},
                {0, {}, 2},
                {0, {}, 3},
                {0, {}, 4},
                {1, {1, 2}, 5},
                {1, {2, 3}, 6},
                {1, {3, 4}, 7},
                {1, {4, 1}, 8},
                {1, {1, 3}, 9},
                {2, {7, 8, 9}, 10},
                {2, {5, 6, 9}, 11}
            };
            const size_t cntss = sizeof(ss)/sizeof(scomplex::simplex);
            c1.add_simplices(ss, cntss);

            //  This is o1        This is o2         This is o3(b)
            //  (value = index)   (values)           (values)
            //
            //  1 ----5---- 2    1 ----5---- 2     1 ----5---- 2
            //  |\          |    |\          |     |\          |
            //  | \     11  |    | \     11  |     | \     13  |
            //  |   \       |    |   \       |     |   \       |
            //  |     \     |    |     \     |     |     \     |
            //  8      9    6    8      9    12    8      9    12
            //  |       \   |    |       \   |     |       \   |
            //  |  10    \  |    |  10    \  |     |  10    \  |
            //  |         \ |    |         \ |     |         \ |
            //  |          \|    |          \|     |          \|
            //  4 ----7---- 3    4 ----7---- 3     4 ----7---- 3

            c2 = c1;
            c2.simplices[6].value = 12;

            c3 = c2;
            c3.simplices[11].value = 13;

            o1.reset();
            o2.reset();
            o3.reset();
            o3b.reset();
            o3b.make_monotone_filtration();
        }

        virtual void tear_down() {
        }

        void test_is_complex() {
            TEST_ASSERT(c1.is_complex());
            TEST_ASSERT(c2.is_complex());
            TEST_ASSERT(c3.is_complex());
        }

        void test_is_monotoneComplex() {
            TEST_ASSERT(c1.is_monotone());
            TEST_ASSERT(!c2.is_monotone());
            TEST_ASSERT(c3.is_monotone());
        }

        void test_is_order_filtration() {
            TEST_ASSERT(o1.is_filtration());
            TEST_ASSERT(o2.is_filtration());
            TEST_ASSERT(o3.is_filtration());
            TEST_ASSERT(o3b.is_filtration());
        }

        void test_is_order_monotone() {
            TEST_ASSERT(o1.is_monotone());
            TEST_ASSERT(!o2.is_monotone());
            TEST_ASSERT(!o3.is_monotone());
            TEST_ASSERT(o3b.is_monotone());
        }

        void test_boundary_matrix() {
            bm mat1 = o1.get_boundary_matrix();
            bm mat1e(c1.size());
            uint32_t mat1e_coords[][2] = {
                    {0, 1}, {0, 2}, {0, 3}, {0, 4}, {1, 5}, {2, 5}, {2, 6}, {3, 6}, {3, 7}, {4, 7}, {1, 8}, {4, 8},
                    {1, 9}, {3, 9}, {7, 10}, {8, 10}, {9, 10}, {5, 11}, {6, 11}, {9, 11}
            };
            mat1e.set_all(mat1e_coords, sizeof(mat1e_coords)/(2*sizeof(uint32_t)), true);
            TEST_ASSERT(mat1 == mat1e);


            bm mat3b = o3b.get_boundary_matrix();
            bm mat3be(c1.size());
            uint32_t mat3be_coords[][2] = {
                    {0, 1}, {0, 2}, {0, 3}, {0, 4}, {1, 5}, {2, 5}, {3, 6}, {4, 6}, {1, 7}, {4, 7}, {1, 8}, {3, 8},
                    {6, 9}, {7, 9}, {8, 9}, {2, 10}, {3, 10}, {5, 11}, {8, 11}, {10, 11}
            };
            mat3be.set_all(mat3be_coords, sizeof(mat3be_coords)/(2*sizeof(uint32_t)), true);
            TEST_ASSERT(mat3b == mat3be);

            //std::cout << mat3b << std::endl << std::endl;
            //std::cout << mat3be << std::endl << std::endl;
            //std::cout << ((bm::colbase) mat3b) << std::endl << std::endl;
            //std::cout << ((bm::rowbase) mat3b) << std::endl << std::endl;
        }
};

#endif
